// Course:    IE 170/171, Laboratory 12
// File name: main.cpp
// Author:    Meredith Somers
// Revision:  scott, asm4: Spring/05
// Purpose:   Locate a search string in Text using different algorithms
// Usage:     executable inputFileName [SearchStringListFilename]
//            For MS VS6.0 user:
//            (1) Create a project, add this file.
//            (2) Open Project menu --> Settings --> Debug, 
//                type data file name (input.txt) into Program Arguments
//                column. 
//#############################################################################

#include "StringMatch.hpp"

#include <ctime>
#include <fstream>

int main(int argc, char * argv[])
{
  //Declare variables
  string P;			//Pattern to be found
  string algo;			//Algorithm to be used

  if(argc < 2) {
    cout<<"Usage:  executable inputFile [SearchStringsListFile]" <<endl;
    return 1;			//Exit abnormally
  }
  
  ifstream is(argv[1]);
  if(!is){
    cout<<"main(): Unable to open the text file! Exiting ..."<<endl;
    exit(1);
  }

  if (argc == 2) {
    // In this mode the user manually enters the search string

    StringMatch sm(is);

    //Begin continuous loop
    while(true) {
      //Prompt the user for a search term(P)
      cout << "Please enter a search term";
      cout << ", or enter \"exit\" or \"quit\" to quit: ";
      cin  >> P;
      if (strcmp(P.c_str(),"quit") == 0 || strcmp(P.c_str(),"exit") == 0) {
	break;
      }

      //Prompt the user for an algorithm
      while (true) {
	cout << "Please enter 'naive' for Naive Approach, ";
	cout << "'rk' for Rabin-Karp, or 'kmp' for Knuth-Morris-Pratt: ";
	cin  >> algo;
	if (strcmp(algo.c_str(),"naive") == 0 || strcmp(algo.c_str(),"rk") == 0
	    || (strcmp(algo.c_str(),"kmp") == 0)) {
	  break;
	}
      }
		
      //Set-up and start timer
      clock_t	start(0), finish(0); //Used to count running time
      double	runTime(0);
		

      //Search for the target word
      cout<< "\nStarting search ..."<<endl;

      if (strcmp(algo.c_str(),"naive") == 0) {
	//Start timing
	start = clock();	//Start to count search time
	sm.NaiveMatcher(P);

	//Stop timing
	finish = clock();	//Finish counting search time
      //Report search time
      runTime = (finish - start)/(CLOCKS_PER_SEC / 1000.0); //in millseconds

      cout << "Naive Search took "<<runTime<<" milliseconds"<<endl<<endl;

      }else if (strcmp(algo.c_str(),"rk") == 0) {	
	//Start timing
	start = clock();	//Start to count search time	
			
	int d = 256;		//all 2^8 ascii characters are allowed 
	int q = 257;		//prime number greater than d
	sm.RabinKarpMatcher(P, d, q);

	//Stop timing
	finish = clock();	//Finish counting search time
      //Report search time
      runTime = (finish - start)/(CLOCKS_PER_SEC / 1000.0); //in millseconds

      cout << "Rabin-Karp Search took "<<runTime<<" milliseconds"<<endl<<endl;
      }
      else if (strcmp(algo.c_str(),"kmp") == 0) {
	//Start timing
	start = clock();	//Start to count search time
	sm.KMPMatcher(P);

	//Stop timing
	finish = clock();	//Finish counting search time
      //Report search time
      runTime = (finish - start)/(CLOCKS_PER_SEC / 1000.0); //in millseconds

      cout << "Knuth-Morris-Pratt Search took "<<runTime
	   <<" milliseconds"<<endl<<endl;
      }
    }
  }

  // if there are two arguments
  // In this mode the user just enters the algo and the program searches for
  // strings contained in argv[2]  
  else {
    ifstream is2(argv[2]);
    if(!is2){
      cout << "main(): Unable to open the Search Strings File!";
      cout << "Exiting ..." << endl; 
      exit(1);
    }

    int numSearchStrings(0);	// number of search strings in file argv[2]
    StringMatch sm(is);

    is2 >> numSearchStrings;
    //Prompt the user for an algorithm
    while (true) {
      cout << "Please enter 'naive' for Naive Approach, ";
      cout << "'rk' for Rabin-Karp, or 'kmp' for Knuth-Morris-Pratt: ";
      cin  >> algo;
      if (strcmp(algo.c_str(),"naive") == 0 || strcmp(algo.c_str(),"rk") == 0
	  || (strcmp(algo.c_str(),"kmp") == 0)) {
	break;
      }
    }
		
    //Set-up and start timer
    clock_t	start(0), finish(0);	//Used to count running time
    double	runTime(0);
    start = clock();		//Start to count search time
		
    if (strcmp(algo.c_str(),"naive") == 0) {
      //Start timing
      for (int i=0; i<numSearchStrings; i++) {
	is2 >> P;
	sm.NaiveMatcher(P);
      }
      //Stop timing
      finish = clock();		//Finish counting search time
      //Report search time
      runTime = (finish - start)/(CLOCKS_PER_SEC / 1000.0); //in millseconds
      cout << "Naive Search took "<<runTime<<" milliseconds"<<endl<<endl;
    }
    
    else if (strcmp(algo.c_str(),"rk") == 0) {	
      int d = 256;		//all 2^8 ascii characters are allowed 
      int q = 257;		//prime number greater than d
      for (int i=0; i<numSearchStrings; i++) {
	is2 >> P;
	sm.RabinKarpMatcher(P, d, q);
      }

      //Stop timing
      finish = clock();		//Finish counting search time
      //Report search time
      runTime = (finish - start)/(CLOCKS_PER_SEC / 1000.0); //in millseconds
      cout<<"Rabin-Karp Search took "<<runTime<<" milliseconds"
	  <<endl<<endl;
    }

    else if (strcmp(algo.c_str(),"kmp") == 0) {
      for (int i=0; i<numSearchStrings; i++) {
	is2 >> P;
	sm.KMPMatcher(P);
      }

      //Stop timing
      finish = clock();		//Finish counting search time
      //Report search time
      runTime = (finish - start)/(CLOCKS_PER_SEC / 1000.0); //in millseconds
      cout<<algo<<"Knuth-Morris-Pratt Search took "<<runTime<<" milliseconds"
	  <<endl << endl;
    }
  }
  return 0;
}

