/*
  File:    queueNumeric.cpp
  Author:  Matthew Galati, Yan Xu, Ted Ralphs, Andrew Ross
  Date:    04/07/03
  Course:  IE 170, Lab 9
*/

/*------------------------------------------------------------------------*/
#include "queueNumeric.hpp"
#include <iostream>
#include <cmath>
using namespace std;

/*------------------------------------------------------------------------*/
QueueNumeric::QueueNumeric()
  : pArrRate(0.0), pSvcRate(0.0), serversNum(0), 
    systemSize(0), probArray(0), trafficPower(0), nFactorial(0) {
}

/*------------------------------------------------------------------------*/
QueueNumeric::QueueNumeric(double pArrR, double pSvcR, int aServ, int serv)
  : pArrRate(pArrR), pSvcRate(pSvcR), serversNum(aServ), 
    systemSize(serv), probArray(0), trafficPower(0), nFactorial(0) {
}

/*------------------------------------------------------------------------*/
int QueueNumeric::errorCheck(){
  if(serversNum > systemSize){
    errorMessage(1);
    return 1;
  }

  if(pArrRate<= 0.0){
    errorMessage(2);
    return 2;
  }

  if(pSvcRate <= 0.0){
    errorMessage(3);
    return 3;
  }
  return 0;
}

/*------------------------------------------------------------------------*/
void QueueNumeric::initialize(){
  probArray      = new double[systemSize - serversNum + 1];
  trafficPower   = new double[systemSize + 1];
  nFactorial     = new double[serversNum + 1];
}

/*------------------------------------------------------------------------*/
void QueueNumeric::errorMessage(const int errorCode){
  switch(errorCode){
  case 1:
    cout << "ERROR: number of available servers > number of total systemSize" 
	 << endl;
    break;
  case 2:
    cout << "ERROR: pArrRate <= 0" << endl;
    break;
  case 3:
    cout << "ERROR: pSvcRate <= 0" << endl;
    break;
  default:
    cout << "ERROR: Unknown" << endl;
  };
}

/*------------------------------------------------------------------------*/
QueueNumeric::~QueueNumeric(){
  if(probArray != 0){
    delete [] probArray;
    probArray = 0;
  }
  if(trafficPower   != 0){
    delete [] trafficPower;
    trafficPower = 0;
  }
  if(nFactorial     != 0){
    delete [] nFactorial;
    nFactorial = 0;
  }
}

/*------------------------------------------------------------------------*/
double QueueNumeric::traffic(){
  return pArrRate* (1.0 / pSvcRate);
}

/*------------------------------------------------------------------------*/
double QueueNumeric::relativeUtilization(){
  return serversNum == 0 ? 1.0 : traffic()/serversNum;
}

/*------------------------------------------------------------------------*/
void QueueNumeric::calculateFactorials(){
  //Use this function to store calculations for future use
}

/*------------------------------------------------------------------------*/
void QueueNumeric::calculateTrafficPowers(){
  //Use this function to store calculations for future use
}

/*------------------------------------------------------------------------*/
void QueueNumeric::MMSK_O(){

  //Check for errors in the input data
  if (errorCheck()) return;
  initialize();

  //Use these to store calculations for future use.
  calculateFactorials();
  calculateTrafficPowers();
  
  //Fill in the calculations here
  double Pd(0.0);
  output(probArray[systemSize], Pd);
}

/*------------------------------------------------------------------------*/
void QueueNumeric::output(const double & Pb, const double & Pd, 
			  ostream * os){
  (*os) << pArrRate<< "\t" << pSvcRate << "\t" << serversNum  << "\t"
	<< systemSize << "\t\t" << Pb << "\t" << Pd << endl;
}
